"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const DefaultRegistry = require("undertaker-registry");
const asyncDone = require("async-done");
const taskContext_1 = require("./taskContext");
const taskTracker_1 = require("./taskTracker");
const clean_1 = require("./tasks/clean");
const statics_1 = require("./tasks/statics");
const tsLint_1 = require("./tasks/tsLint");
const compiler_1 = require("./tasks/compiler");
const serverTest_1 = require("./tasks/serverTest");
const server_1 = require("./tasks/server");
const webpackDevServer_1 = require("./tasks/webpackDevServer");
class Registry extends DefaultRegistry {
    constructor(config) {
        super();
        assert(typeof config.rootPath === 'string', 'rootPath must be defined in Registry options');
        this.context = new taskContext_1.TaskContext(config);
    }
    init(gulp) {
        super.init(gulp);
        const { task, parallel, series } = gulp;
        /* tslint:disable-next-line */
        const tracker = new taskTracker_1.TaskTracker(gulp);
        const server = new server_1.Server(this.context);
        const cleaner = new clean_1.Cleaner(this.context);
        const statics = new statics_1.Statics(this.context);
        const tslint = new tsLint_1.TsLinter(this.context);
        const compiler = new compiler_1.ServerCompiler(this.context);
        const serverTest = new serverTest_1.ServerTest(this.context, server);
        const webpackDevServer = new webpackDevServer_1.WebpackDevServer(this.context);
        // lazily get task dependencies so overridden task definitions are as part of these chains
        const inject = taskName => {
            const fn = done => asyncDone(task(taskName), done);
            fn.displayName = `[injected] ${taskName}`;
            return fn;
        };
        task('clean:client', cleaner.cleanClient);
        task('clean:server', cleaner.cleanServer);
        task('clean', parallel(inject('clean:client'), inject('clean:server')));
        task('statics:build', statics.build);
        task('tslint:tasks', tslint.lintTasks);
        task('tslint:server', tslint.lintServer);
        task('tslint:client', tslint.lintClient);
        task('tslint', parallel(inject('tslint:client'), inject('tslint:server')));
        task('client:prebuild', parallel(inject('tslint:client'), series(inject('clean:client'), inject('statics:build'))));
        task('client:build', series(inject('client:prebuild')
        /* TODO: add webpack here */ ));
        task('client:devServer', series(inject('client:prebuild'), webpackDevServer.serve));
        task('server:precompile', done => done());
        task('server:compile', series(inject('server:precompile'), compiler.compile));
        task('server:test:single', series(inject('server:compile'), serverTest.single));
        task('server:test', series(parallel(inject('server:test:single'), compiler.watch), serverTest.continuous));
        task('server:run', parallel(inject('tslint:server'), parallel(compiler.watch, serverTest.applyHooks), series(inject('clean:server'), inject('server:compile'), server.serve)));
        task('dev', parallel(inject('client:devServer'), inject('server:run')));
        task('default', inject('dev'));
    }
}
exports.Registry = Registry;
